@@ -7,15 +7,6 @@ if sys.version_info[:2] < (2, 3):
     sys.stderr.write("Sorry, Cython requires Python 2.3 or later\n")
     sys.exit(1)
 
-try:
-    set
-except NameError:
-    # Python 2.3
-    from sets import Set as set
-
-import itertools
-
-import Code
 import Errors
 # Do not import Parsing here, import it when needed, because Parsing imports
 # Nodes, which globally needs debug command line options initialized to set a
@@ -24,11 +15,9 @@ import Errors
 # import Parsing
 import Version
 from Scanning import PyrexScanner, FileSourceDescriptor
-from Errors import PyrexError, CompileError, InternalError, AbortError, error, warning
-from Symtab import BuiltinScope, ModuleScope
+from Errors import PyrexError, CompileError, error, warning
+from Symtab import ModuleScope
 from Cython import Utils
-from Cython.Utils import open_new_file, replace_suffix
-import DebugFlags
 import Options
 
 module_name_pattern = re.compile(r"[A-Za-z_][A-Za-z0-9_]*(\.[A-Za-z_][A-Za-z0-9_]*)*$")
@@ -71,9 +60,8 @@ class Context(object):
 
         import Builtin, CythonScope
         self.modules = {"__builtin__" : Builtin.builtin_scope}
-        cyscope = CythonScope.create_cython_scope(
-                self, create_testscope=create_testscope)
-        self.modules["cython"] = self.cython_scope = cyscope
+        self.cython_scope = CythonScope.create_cython_scope(self)
+        self.modules["cython"] = self.cython_scope
         self.include_directories = include_directories
         self.future_directives = set()
         self.compiler_directives = compiler_directives
@@ -194,7 +182,7 @@ class Context(object):
         # the directory containing the source file is searched first
         # for a dotted filename, and its containing package root
         # directory is searched first for a non-dotted filename.
-        pxd = self.search_include_directories(qualified_name, ".pxd", pos)
+        pxd = self.search_include_directories(qualified_name, ".pxd", pos, sys_path=True)
         if pxd is None: # XXX Keep this until Includes/Deprecated is removed
             if (qualified_name.startswith('python') or
                 qualified_name in ('stdlib', 'stdio', 'stl')):
@@ -233,13 +221,16 @@ class Context(object):
         return path
 
     def search_include_directories(self, qualified_name, suffix, pos,
-                                   include=False):
+                                   include=False, sys_path=False):
         # Search the list of include directories for the given
         # file name. If a source file position is given, first
         # searches the directory containing that file. Returns
         # None if not found, but does not report an error.
         # The 'include' option will disable package dereferencing.
+        # If 'sys_path' is True, also search sys.path.
         dirs = self.include_directories
+        if sys_path:
+            dirs = dirs + sys.path
         if pos:
             file_desc = pos[0]
             if not isinstance(file_desc, FileSourceDescriptor):
@@ -366,10 +357,28 @@ class Context(object):
                 tree = Parsing.p_module(s, pxd, full_module_name)
             finally:
                 f.close()
-        except UnicodeDecodeError, msg:
+        except UnicodeDecodeError, e:
             #import traceback
             #traceback.print_exc()
-            error((source_desc, 0, 0), "Decoding error, missing or incorrect coding=<encoding-name> at top of source (%s)" % msg)
+            line = 1
+            column = 0
+            msg = e.args[-1]
+            position = e.args[2]
+            encoding = e.args[0]
+
+            for idx, c in enumerate(open(source_filename, "rb").read()):
+                if c in (ord('\n'), '\n'):
+                    line += 1
+                    column = 0
+                if idx == position:
+                    break
+
+                column += 1
+
+            error((source_desc, line, column),
+                  "Decoding error, missing or incorrect coding=<encoding-name> "
+                  "at top of source (cannot decode with encoding %r: %s)" % (encoding, msg))
+
         if Errors.num_errors > 0:
             raise CompileError
         return tree
@@ -435,7 +444,7 @@ def create_default_resultobj(compilation_source, options):
 
 def run_pipeline(source, options, full_module_name = None):
     import Pipeline
-    # Set up context
+
     context = options.create_context()
 
     # Set up source object
@@ -443,6 +452,7 @@ def run_pipeline(source, options, full_module_name = None):
     abs_path = os.path.abspath(source)
     source_ext = os.path.splitext(source)[1]
     full_module_name = full_module_name or context.extract_module_name(source, options)
+
     if options.relative_path_in_code_position_comments:
         rel_path = full_module_name.replace('.', os.sep) + source_ext
         if not abs_path.endswith(rel_path):
@@ -525,7 +535,7 @@ class CompilationOptions(object):
 
     def create_context(self):
         return Context(self.include_path, self.compiler_directives,
-                      self.cplus, self.language_level, options=self)
+                       self.cplus, self.language_level, options=self)
 
 
 class CompilationResult(object):
@@ -589,7 +599,8 @@ def compile_multiple(sources, options):
     a CompilationResultSet. Performs timestamp checking and/or recursion
     if these are specified in the options.
     """
-    context = options.create_context()
+    # run_pipeline creates the context
+    # context = options.create_context()
     sources = [os.path.abspath(source) for source in sources]
     processed = set()
     results = CompilationResultSet()
@@ -696,4 +707,5 @@ default_options = dict(
     c_line_in_traceback = True,
     language_level = 2,
     gdb_debug = False,
+    compile_time_env = None,
 )
