@@ -67,11 +67,15 @@ old_style_globals = False
 cimport_from_pyx = False
 
 
+# max # of dims for buffers -- set lower than number of dimensions in numpy, as
+# slices are passed by value and involve a lot of copying
+buffer_max_dims = 8
 
 # Declare compiler directives
 directive_defaults = {
     'boundscheck' : True,
     'nonecheck' : False,
+    'initializedcheck' : True,
     'embedsignature' : False,
     'locals' : {},
     'auto_cpdef': False,
@@ -92,17 +96,19 @@ directive_defaults = {
     'autotestdict.all': False,
     'language_level': 2,
     'fast_getattr': False, # Undocumented until we come up with a better way to handle this everywhere.
-    'py2_import': False, # For backward compatibility of Cython's source code
+    'py2_import': False, # For backward compatibility of Cython's source code in Py3 source mode
 
     'warn': None,
     'warn.undeclared': False,
     'warn.unreachable': True,
     'warn.maybe_uninitialized': False,
-    'warn.unreachable': True,
     'warn.unused': False,
     'warn.unused_arg': False,
     'warn.unused_result': False,
 
+# optimizations
+    'optimize.inline_defnode_calls': False,
+
 # remove unreachable code
     'remove_unreachable': True,
 
@@ -133,6 +139,7 @@ directive_types = {
     'cfunc' : None, # decorators do not take directive value
     'ccall' : None,
     'cclass' : None,
+    'returns' : type,
     }
 
 for key, val in directive_defaults.items():
@@ -141,7 +148,7 @@ for key, val in directive_defaults.items():
 
 directive_scopes = { # defaults to available everywhere
     # 'module', 'function', 'class', 'with statement'
-    'final' : ('cclass',),   # add 'method' in the future
+    'final' : ('cclass', 'function'),
     'internal' : ('cclass',),
     'autotestdict' : ('module',),
     'autotestdict.all' : ('module',),
