@@ -138,15 +138,23 @@ class ResultRefNode(AtomicExprNode):
         # nothing to do here
         return self
 
+    def type_dependencies(self, env):
+        if self.expression:
+            return self.expression.type_dependencies(env)
+        else:
+            return ()
+
     def analyse_types(self, env):
         if self.expression is not None:
             self.type = self.expression.type
 
     def infer_type(self, env):
-        if self.expression is not None:
-            return self.expression.infer_type(env)
         if self.type is not None:
             return self.type
+        if self.expression is not None:
+            if self.expression.type is not None:
+                return self.expression.type
+            return self.expression.infer_type(env)
         assert False, "cannot infer type of ResultRefNode"
 
     def may_be_none(self):
