@@ -28,6 +28,10 @@ class _EmptyDecoratorAndManager(object):
 
 cclass = ccall = cfunc = _EmptyDecoratorAndManager()
 
+returns = lambda type_arg: _EmptyDecoratorAndManager()
+
+final = internal = _empty_decorator
+
 def inline(f, *args, **kwds):
   if isinstance(f, basestring):
     from Cython.Build.Inline import cython_inline
@@ -66,7 +70,8 @@ def sizeof(arg):
     return 1
 
 def typeof(arg):
-    return type(arg)
+    return arg.__class__.__name__
+    # return type(arg)
 
 def address(arg):
     return pointer(type(arg))([arg])
@@ -115,7 +120,7 @@ class PointerType(CythonType):
             self._items = [cast(self._basetype, a) for a in value._items]
         elif isinstance(value, list):
             self._items = [cast(self._basetype, a) for a in value]
-        elif value is None or value is 0:
+        elif value is None or value == 0:
             self._items = []
         else:
             raise ValueError
@@ -138,6 +143,9 @@ class PointerType(CythonType):
         else:
             return not self._items and not value._items
 
+    def __repr__(self):
+        return "%s *" % (self._basetype,)
+
 class ArrayType(PointerType):
 
     def __init__(self):
@@ -221,35 +229,56 @@ def union(**members):
 
 class typedef(CythonType):
 
-    def __init__(self, type):
+    def __init__(self, type, name=None):
         self._basetype = type
+        self.name = name
 
     def __call__(self, *arg):
         value = cast(self._basetype, *arg)
         return value
 
+    def __repr__(self):
+        return self.name or str(self._basetype)
+
+class _FusedType(CythonType):
+    pass
 
 
-py_int = int
+def fused_type(*args):
+    if not args:
+        raise TypeError("Expected at least one type as argument")
+
+    # Find the numeric type with biggest rank if all types are numeric
+    rank = -1
+    for type in args:
+        if type not in (py_int, py_long, py_float, py_complex):
+            break
+
+        if type_ordering.index(type) > rank:
+            result_type = type
+    else:
+        return result_type
+
+    # Not a simple numeric type, return a fused type instance. The result
+    # isn't really meant to be used, as we can't keep track of the context in
+    # pure-mode. Casting won't do anything in this case.
+    return _FusedType()
+
+
+def _specialized_from_args(signatures, args, kwargs):
+    "Perhaps this should be implemented in a TreeFragment in Cython code"
+    raise Exception("yet to be implemented")
+
+
+py_int = typedef(int, "int")
 try:
-    py_long = long
+    py_long = typedef(long, "long")
 except NameError: # Py3
-    py_long = int
-py_float = float
-py_complex = complex
+    py_long = typedef(int, "long")
+py_float = typedef(float, "float")
+py_complex = typedef(complex, "double complex")
 
 
-try:
-    # Python 3
-    from builtins import set, frozenset
-except ImportError:
-    try:
-        # Python 2.4+
-        from __builtin__ import set, frozenset
-    except ImportError:
-        # Py 2.3
-        from sets import Set as set, ImmutableSet as frozenset
-
 # Predefined types
 
 int_types = ['char', 'short', 'Py_UNICODE', 'int', 'long', 'longlong', 'Py_ssize_t', 'size_t']
@@ -257,30 +286,43 @@ float_types = ['longdouble', 'double', 'float']
 complex_types = ['longdoublecomplex', 'doublecomplex', 'floatcomplex', 'complex']
 other_types = ['bint', 'void']
 
+to_repr = {
+    'longlong': 'long long',
+    'longdouble': 'long double',
+    'longdoublecomplex': 'long double complex',
+    'doublecomplex': 'double complex',
+    'floatcomplex': 'float complex',
+}.get
+
 gs = globals()
 
 for name in int_types:
-    gs[name] = typedef(py_int)
+    reprname = to_repr(name, name)
+    gs[name] = typedef(py_int, reprname)
     if name != 'Py_UNICODE' and not name.endswith('size_t'):
-        gs['u'+name] = typedef(py_int)
-        gs['s'+name] = typedef(py_int)
+        gs['u'+name] = typedef(py_int, "unsigned " + reprname)
+        gs['s'+name] = typedef(py_int, "signed " + reprname)
 
 for name in float_types:
-    gs[name] = typedef(py_float)
+    gs[name] = typedef(py_float, to_repr(name, name))
 
 for name in complex_types:
-    gs[name] = typedef(py_complex)
+    gs[name] = typedef(py_complex, to_repr(name, name))
 
-bint = typedef(bool)
-void = typedef(int)
+bint = typedef(bool, "bint")
+void = typedef(int, "void")
 
 for t in int_types + float_types + complex_types + other_types:
     for i in range(1, 4):
         gs["%s_%s" % ('p'*i, t)] = globals()[t]._pointer(i)
 
-void = typedef(None)
+void = typedef(None, "void")
 NULL = p_void(0)
 
+integral = floating = numeric = _FusedType()
+
+type_ordering = [py_int, py_long, py_float, py_complex]
+
 class CythonDotParallel(object):
     """
     The cython.parallel module.
