@@ -5,15 +5,16 @@ import sys
 import re
 import gc
 import locale
-import codecs
 import shutil
 import time
 import unittest
 import doctest
 import operator
+import subprocess
 import tempfile
-import warnings
 import traceback
+import warnings
+
 try:
     from StringIO import StringIO
 except ImportError:
@@ -82,6 +83,7 @@ def update_numpy_extension(ext):
     ext.include_dirs.append(numpy.get_include())
 
 def update_openmp_extension(ext):
+    ext.openmp = True
     language = ext.language
 
     if language == 'cpp':
@@ -95,6 +97,8 @@ def update_openmp_extension(ext):
         ext.extra_compile_args.extend(compile_flags.split())
         ext.extra_link_args.extend(link_flags.split())
         return ext
+    elif sys.platform == 'win32':
+        return ext
 
     return EXCLUDE_EXT
 
@@ -109,35 +113,34 @@ def get_openmp_compiler_flags(language):
         cc = sysconfig.get_config_var('CXX')
     else:
         cc = sysconfig.get_config_var('CC')
-    if not cc: return None # Windows?
+
+    if not cc:
+        return None # Windows?
 
     # For some reason, cc can be e.g. 'gcc -pthread'
     cc = cc.split()[0]
 
+    # Force english output
+    env = os.environ.copy()
+    env['LC_MESSAGES'] = 'C'
+
     matcher = re.compile(r"gcc version (\d+\.\d+)").search
     try:
-        import subprocess
-    except ImportError:
-        try:
-            in_, out_err = os.popen4([cc, "-v"])
-        except EnvironmentError:
-            warnings.warn("Unable to find the %s compiler: %s: %s" %
-                          (language, os.strerror(sys.exc_info()[1].errno), cc))
-            return None
-        output = out_err.read()
-    else:
-        try:
-            p = subprocess.Popen([cc, "-v"], stderr=subprocess.PIPE)
-        except EnvironmentError:
-            # Be compatible with Python 3
-            warnings.warn("Unable to find the %s compiler: %s: %s" %
-                          (language, os.strerror(sys.exc_info()[1].errno), cc))
-            return None
-        _, output = p.communicate()
+        p = subprocess.Popen([cc, "-v"], stderr=subprocess.PIPE, env=env)
+    except EnvironmentError:
+        # Be compatible with Python 3
+        warnings.warn("Unable to find the %s compiler: %s: %s" %
+                      (language, os.strerror(sys.exc_info()[1].errno), cc))
+        return None
+    _, output = p.communicate()
 
     output = output.decode(locale.getpreferredencoding() or 'ASCII', 'replace')
 
-    compiler_version = matcher(output).group(1)
+    gcc_version = matcher(output)
+    if not gcc_version:
+        return None # not gcc - FIXME: do something about other compilers
+
+    compiler_version = gcc_version.group(1)
     if compiler_version and compiler_version.split('.') >= ['4', '2']:
         return '-fopenmp', '-fopenmp'
 
@@ -172,9 +175,17 @@ VER_DEP_MODULES = {
                                           ]),
     (2,6) : (operator.lt, lambda x: x in ['run.print_function',
                                           'run.cython3',
+                                          'run.property_decorator_T593', # prop.setter etc.
                                           'run.generators_py', # generators, with statement
                                           'run.pure_py', # decorators, with statement
                                           'run.purecdef',
+                                          'run.struct_conversion',
+                                          'run.cythonarray',
+                                          'run.memslice',
+                                          'run.numpy_memoryview',
+                                          'run.memoryviewattrs',
+                                          'run.memoryview',
+                                          'compile.memview_declaration',
                                           ]),
     (2,7) : (operator.lt, lambda x: x in ['run.withstat_py', # multi context with statement
                                           'run.yield_inside_lambda',
@@ -252,6 +263,8 @@ class build_ext(_build_ext):
                 compiler_obj.compiler_so.remove('-Wstrict-prototypes')
             if CCACHE:
                 compiler_obj.compiler_so = CCACHE + compiler_obj.compiler_so
+            if getattr(ext, 'openmp', None) and compiler_obj.compiler_type == 'msvc':
+                ext.extra_compile_args.append('/openmp')
         except Exception:
             pass
         _build_ext.build_extension(self, ext)
@@ -286,8 +299,8 @@ class ErrorWriter(object):
 
 class TestBuilder(object):
     def __init__(self, rootdir, workdir, selectors, exclude_selectors, annotate,
-                 cleanup_workdir, cleanup_sharedlibs, with_pyregr, cython_only,
-                 languages, test_bugs, fork, language_level):
+                 cleanup_workdir, cleanup_sharedlibs, cleanup_failures,
+                 with_pyregr, cython_only, languages, test_bugs, fork, language_level):
         self.rootdir = rootdir
         self.workdir = workdir
         self.selectors = selectors
@@ -295,6 +308,7 @@ class TestBuilder(object):
         self.annotate = annotate
         self.cleanup_workdir = cleanup_workdir
         self.cleanup_sharedlibs = cleanup_sharedlibs
+        self.cleanup_failures = cleanup_failures
         self.with_pyregr = with_pyregr
         self.cython_only = cython_only
         self.languages = languages
@@ -411,6 +425,7 @@ class TestBuilder(object):
                           annotate=self.annotate,
                           cleanup_workdir=self.cleanup_workdir,
                           cleanup_sharedlibs=self.cleanup_sharedlibs,
+                          cleanup_failures=self.cleanup_failures,
                           cython_only=self.cython_only,
                           fork=self.fork,
                           language_level=self.language_level,
@@ -419,8 +434,8 @@ class TestBuilder(object):
 class CythonCompileTestCase(unittest.TestCase):
     def __init__(self, test_directory, workdir, module, language='c',
                  expect_errors=False, annotate=False, cleanup_workdir=True,
-                 cleanup_sharedlibs=True, cython_only=False, fork=True,
-                 language_level=2, warning_errors=False):
+                 cleanup_sharedlibs=True, cleanup_failures=True, cython_only=False,
+                 fork=True, language_level=2, warning_errors=False):
         self.test_directory = test_directory
         self.workdir = workdir
         self.module = module
@@ -429,6 +444,7 @@ class CythonCompileTestCase(unittest.TestCase):
         self.annotate = annotate
         self.cleanup_workdir = cleanup_workdir
         self.cleanup_sharedlibs = cleanup_sharedlibs
+        self.cleanup_failures = cleanup_failures
         self.cython_only = cython_only
         self.fork = fork
         self.language_level = language_level
@@ -442,6 +458,7 @@ class CythonCompileTestCase(unittest.TestCase):
         from Cython.Compiler import Options
         self._saved_options = [ (name, getattr(Options, name))
                                 for name in ('warning_errors', 'error_on_unknown_names') ]
+        self._saved_default_directives = Options.directive_defaults.items()
         Options.warning_errors = self.warning_errors
 
         if self.workdir not in sys.path:
@@ -451,6 +468,7 @@ class CythonCompileTestCase(unittest.TestCase):
         from Cython.Compiler import Options
         for name, value in self._saved_options:
             setattr(Options, name, value)
+        Options.directive_defaults = dict(self._saved_default_directives)
 
         try:
             sys.path.remove(self.workdir)
@@ -460,16 +478,17 @@ class CythonCompileTestCase(unittest.TestCase):
             del sys.modules[self.module]
         except KeyError:
             pass
-        cleanup_c_files = WITH_CYTHON and self.cleanup_workdir
-        cleanup_lib_files = self.cleanup_sharedlibs
+        cleanup = self.cleanup_failures or self.success
+        cleanup_c_files = WITH_CYTHON and self.cleanup_workdir and cleanup
+        cleanup_lib_files = self.cleanup_sharedlibs and cleanup
         if os.path.exists(self.workdir):
             for rmfile in os.listdir(self.workdir):
                 if not cleanup_c_files:
-                    if rmfile[-2:] in (".c", ".h") or rmfile[-4:] == ".cpp":
+                    if (rmfile[-2:] in (".c", ".h") or
+                        rmfile[-4:] == ".cpp" or
+                        rmfile.endswith(".html")):
                         continue
-                if not cleanup_lib_files and rmfile.endswith(".so") or rmfile.endswith(".dll"):
-                    continue
-                if self.annotate and rmfile.endswith(".html"):
+                if not cleanup_lib_files and (rmfile.endswith(".so") or rmfile.endswith(".dll")):
                     continue
                 try:
                     rmfile = os.path.join(self.workdir, rmfile)
@@ -483,7 +502,9 @@ class CythonCompileTestCase(unittest.TestCase):
             os.makedirs(self.workdir)
 
     def runTest(self):
+        self.success = False
         self.runCompileTest()
+        self.success = True
 
     def runCompileTest(self):
         self.compile(self.test_directory, self.module, self.workdir,
@@ -588,7 +609,12 @@ class CythonCompileTestCase(unittest.TestCase):
             build_extension.finalize_options()
             if COMPILER:
                 build_extension.compiler = COMPILER
+
             ext_compile_flags = CFLAGS[:]
+            compiler = COMPILER or sysconfig.get_config_var('CC')
+
+            if self.language == 'c' and compiler == 'gcc':
+                ext_compile_flags.extend(['-std=c89', '-pedantic'])
             if  build_extension.compiler == 'mingw32':
                 ext_compile_flags.append('-Wno-format')
             if extra_extension_args is None:
@@ -675,8 +701,13 @@ class CythonRunTestCase(CythonCompileTestCase):
         try:
             self.setUp()
             try:
+                self.success = False
                 self.runCompileTest()
+                failures, errors = len(result.failures), len(result.errors)
                 self.run_tests(result)
+                if failures == len(result.failures) and errors == len(result.errors):
+                    # No new errors...
+                    self.success = True
             finally:
                 check_thread_termination()
         except Exception:
@@ -704,6 +735,7 @@ def run_forked_test(result, run_func, test_name, fork=True):
         gc.collect()
         return
 
+    module_name = test_name.split()[-1]
     # fork to make sure we do not keep the tested module loaded
     result_handle, result_file = tempfile.mkstemp()
     os.close(result_handle)
@@ -858,6 +890,7 @@ class CythonPyregrTestCase(CythonRunTestCase):
         CythonRunTestCase.setUp(self)
         from Cython.Compiler import Options
         Options.error_on_unknown_names = False
+        Options.directive_defaults.update(dict(binding=True, always_allow_keywords=True))
 
     def _run_unittest(self, result, *classes):
         """Run tests from unittest.TestCase-derived classes."""
@@ -957,7 +990,7 @@ def collect_doctests(path, module_prefix, suite, selectors):
                 '#' in filename and not
                 filename.startswith('.') and not
                 filename in blacklist)
-    import doctest, types
+    import doctest
     for dirpath, dirnames, filenames in os.walk(path):
         for dir in list(dirnames):
             if not package_matches(dir):
@@ -1029,6 +1062,7 @@ class EndToEndTest(unittest.TestCase):
         os.chdir(self.old_dir)
 
     def runTest(self):
+        self.success = False
         commands = (self.commands
             .replace("CYTHON", "PYTHON %s" % os.path.join(self.cython_root, 'cython.py'))
             .replace("PYTHON", sys.executable))
@@ -1036,26 +1070,23 @@ class EndToEndTest(unittest.TestCase):
             old_path = os.environ.get('PYTHONPATH')
             os.environ['PYTHONPATH'] = self.cython_syspath + os.pathsep + os.path.join(self.cython_syspath, (old_path or ''))
             for command in commands.split('\n'):
-                if sys.version_info[:2] >= (2,4):
-                    import subprocess
-                    p = subprocess.Popen(commands,
-                                         stderr=subprocess.PIPE,
-                                         stdout=subprocess.PIPE,
-                                         shell=True)
-                    out, err = p.communicate()
-                    res = p.returncode
-                    if res != 0:
-                        print(command)
-                        print(out)
-                        print(err)
-                else:
-                    res = os.system(command)
+                p = subprocess.Popen(commands,
+                                     stderr=subprocess.PIPE,
+                                     stdout=subprocess.PIPE,
+                                     shell=True)
+                out, err = p.communicate()
+                res = p.returncode
+                if res != 0:
+                    print(command)
+                    print(out)
+                    print(err)
                 self.assertEqual(0, res, "non-zero exit status")
         finally:
             if old_path:
                 os.environ['PYTHONPATH'] = old_path
             else:
                 del os.environ['PYTHONPATH']
+        self.success = True
 
 
 # TODO: Support cython_freeze needed here as well.
@@ -1165,7 +1196,11 @@ class TagsSelector:
 class RegExSelector:
     
     def __init__(self, pattern_string):
-        self.pattern = re.compile(pattern_string, re.I|re.U)
+        try:
+            self.pattern = re.compile(pattern_string, re.I|re.U)
+        except re.error:
+            print('Invalid pattern: %r' % pattern_string)
+            raise
 
     def __call__(self, testname, tags=None):
         return self.pattern.search(testname)
@@ -1196,6 +1231,7 @@ def refactor_for_py3(distdir, cy3_dir):
                      recursive-exclude Cython *
                      recursive-include Cython *.py *.pyx *.pxd
                      recursive-include Cython/Debugger/Tests *
+                     recursive-include Cython/Utility *
                      include runtests.py
                      include cython.py
                      ''')
@@ -1238,12 +1274,8 @@ def check_thread_termination(ignore_seen=True):
 
 def subprocess_output(cmd):
     try:
-        try:
-            import subprocess
-        except:
-            return os.popen4(cmd)[1].read()
-        else:
-            return subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT).communicate()[0]
+        p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
+        return p.communicate()[0].decode('UTF-8')
     except OSError:
         return ''
 
@@ -1252,8 +1284,8 @@ def get_version():
     full_version = cython_version
     top = os.path.dirname(os.path.abspath(__file__))
     if os.path.exists(os.path.join(top, '.git')):
+        old_dir = os.getcwd()
         try:
-            old_dir = os.getcwd()
             os.chdir(top)
             head_commit = subprocess_output(['git', 'rev-parse', 'HEAD']).strip()
             version_commit = subprocess_output(['git', 'rev-parse', cython_version]).strip()
@@ -1266,6 +1298,14 @@ def get_version():
             os.chdir(old_dir)
     return full_version
 
+_orig_stdout, _orig_stderr = sys.stdout, sys.stderr
+def flush_and_terminate(status):
+    try:
+        _orig_stdout.flush()
+        _orig_stderr.flush()
+    finally:
+        os._exit(status)
+
 def main():
 
     DISTDIR = os.path.join(os.getcwd(), os.path.dirname(sys.argv[0]))
@@ -1278,6 +1318,9 @@ def main():
     parser.add_option("--no-cleanup-sharedlibs", dest="cleanup_sharedlibs",
                       action="store_false", default=True,
                       help="do not delete the generated shared libary files (allows manual module experimentation)")
+    parser.add_option("--no-cleanup-failures", dest="cleanup_failures",
+                      action="store_false", default=True,
+                      help="enable --no-cleanup and --no-cleanup-sharedlibs for failed tests only")
     parser.add_option("--no-cython", dest="with_cython",
                       action="store_false", default=True,
                       help="do not run the Cython compiler, only the C compiler")
@@ -1352,6 +1395,12 @@ def main():
                       help="working directory")
     parser.add_option("--work-dir", dest="work_dir", default=os.path.join(os.getcwd(), 'BUILD'),
                       help="working directory")
+    parser.add_option("--cython-dir", dest="cython_dir", default=os.getcwd(),
+                      help="Cython installation directory (default: use local source version)")
+    parser.add_option("--debug", dest="for_debugging", default=False, action="store_true",
+                      help="configure for easier use with a debugger (e.g. gdb)")
+    parser.add_option("--pyximport-py", dest="pyximport_py", default=False, action="store_true",
+                      help="use pyximport to automatically compile imported .pyx and .py files")
 
     options, cmd_args = parser.parse_args()
 
@@ -1361,6 +1410,7 @@ def main():
     if sys.version_info[0] >= 3:
         options.doctests = False
         if options.with_cython:
+            sys.path.insert(0, options.cython_dir)
             try:
                 # try if Cython is installed in a Py3 version
                 import Cython.Compiler.Main
@@ -1423,6 +1473,11 @@ def main():
     else:
         sys.stderr.write("Running tests without Cython.\n")
 
+    if options.for_debugging:
+        options.cleanup_workdir = False
+        options.cleanup_sharedlibs = False
+        options.fork = False
+
     if options.with_refnanny:
         from pyximport.pyxbuild import pyx_to_dll
         libpath = pyx_to_dll(os.path.join("Cython", "Runtime", "refnanny.pyx"),
@@ -1449,7 +1504,6 @@ def main():
             if selector.startswith('bugs'):
                 test_bugs = True
 
-    import re
     selectors = [ string_selector(r) for r in cmd_args ]
     if not selectors:
         selectors = [ lambda x, tags=None: True ]
@@ -1502,7 +1556,8 @@ def main():
     if options.filetests and languages:
         filetests = TestBuilder(ROOTDIR, WORKDIR, selectors, exclude_selectors,
                                 options.annotate_source, options.cleanup_workdir,
-                                options.cleanup_sharedlibs, options.pyregr,
+                                options.cleanup_sharedlibs, options.cleanup_failures,
+                                options.pyregr,
                                 options.cython_only, languages, test_bugs,
                                 options.fork, options.language_level)
         test_suite.addTest(filetests.build_suite())
@@ -1512,7 +1567,8 @@ def main():
         if os.path.isdir(sys_pyregr_dir):
             filetests = TestBuilder(ROOTDIR, WORKDIR, selectors, exclude_selectors,
                                     options.annotate_source, options.cleanup_workdir,
-                                    options.cleanup_sharedlibs, True,
+                                    options.cleanup_sharedlibs, options.cleanup_failures,
+                                    True,
                                     options.cython_only, languages, test_bugs,
                                     options.fork, sys.version_info[0])
             sys.stderr.write("Including CPython regression tests in %s\n" % sys_pyregr_dir)
@@ -1525,6 +1581,11 @@ def main():
     else:
         test_runner = unittest.TextTestRunner(verbosity=options.verbosity)
 
+    if options.pyximport_py:
+        from pyximport import pyximport
+        pyximport.install(pyimport=True, build_dir=os.path.join(WORKDIR, '_pyximport'),
+                          load_py_module_on_import_failure=True)
+
     result = test_runner.run(test_suite)
 
     if options.coverage or options.coverage_xml or options.coverage_html:
@@ -1562,7 +1623,7 @@ def main():
         sys.exit(return_code)
     except PendingThreadsError:
         # normal program exit won't kill the threads, do it the hard way here
-        os._exit(return_code)
+        flush_and_terminate(return_code)
 
 if __name__ == '__main__':
     try:
@@ -1575,4 +1636,4 @@ if __name__ == '__main__':
             check_thread_termination(ignore_seen=False)
         except PendingThreadsError:
             # normal program exit won't kill the threads, do it the hard way here
-            os._exit(1)
+            flush_and_terminate(1)
